module ContactCongressParser
  class Action
    attr_reader :name, :data

    def self.create_from_step(step)
      name = step.keys.first
      values = step.values.first

      if values.class == Array
        values.map { |value| self.new(name, value) }
      else
        self.new(name, values)
      end
    end

    def initialize(name, data)
      @name = name
      @data = data
    end

    def to_s
      # don't parse wait and find, they are problematic and (probably ;) unecessary
      ['wait', 'find'].include?(name) ? '' : name + ' ' + send("#{name}_args")
    end

    def visit_args
      '"' + data + '"'
    end

    def find_args
      to_find = data['value'] || data['selector']
      '"' + to_find + '"'
    end

    def fill_in_args
      '"' + escape_quotes(@data['selector']) + '"' +
      ', with: ' +
      field(@data['value'], required: @data['required'])
    end

    def select_args
      field(@data['value'], options: @data['options'], required: @data['required'])
    end
    
    def choose_args
      # some chooses in contact-congress behave differently
      if data['options']
        select_args
      else
        '"' + @data['selector'] + '"'
      end
    end

    def check_args
      '"' + @data['selector'] + '"'
    end

    def uncheck_args
      check_args
    end

    def click_on_args
      to_click_on = data['value'] || data['selector']
      '"' + to_click_on + '"'
    end

  private
    def path(uri)
      URI.parse(uri).path
    end

    def field(name, options={})
      Field.new(name, options).to_s
    end

    def escape_quotes(str)
      str.gsub('"', '\"')
    end
  end
end
