module ContactCongressParser
  class Field
    attr_reader :name, :options, :field_options, :required

    def initialize(name, options={})
      @name = normalize_field_name(name)
      @options = options
    end

    def to_s
      "field(\"#{name}\"#{options_str})"
    end

    def required
      @required ||= case options[:required]
        when 'Yes', true then true
        when 'No', false then false
        when nil then nil
        else raise "Unrecognized options[:required] => #{options[:required]}"
      end
    end

    def normalize_field_name(name)
      if name[0] == '$'
        case name
        when '$ADDRESS_CITY'                then 'City'
        when '$ADDRESS_COUNTY'              then 'County'
        when '$ADDRESS_STATE_FULL'          then 'State'
        when '$ADDRESS_STATE_POSTAL_ABBREV' then 'State'
        when '$ADDRESS_STREET'              then 'Address'
        when '$ADDRESS_STREET_2'            then 'Address Cont.'
        when '$ADDRESS_ZIP4'                then 'ZIP 4'
        when '$ADDRESS_ZIP5'                then 'ZIP'
        when '$ADDRESS_ZIP_PLUS_4'          then 'ZIP+4'
        when '$CAMPAIGN_UUID'               then 'UUID'
        when '$CAPTCHA_SOLUTION'            then 'captcha'
        when '$EMAIL'                       then 'Email Address'
        when '$MESSAGE'                     then 'Message'
        when '$NAME_FIRST'                  then 'First Name'
        when '$NAME_FULL'                   then 'Name'
        when '$NAME_LAST'                   then 'Last Name'
        when '$NAME_PREFIX'                 then 'Title'
        when '$NAME_SUFFIX'                 then 'Suffix'
        when '$ORG_NAME'                    then 'Organization'
        when '$ORG_URL'                     then 'Organization URL'
        when '$PHONE'                       then 'Phone'
        when '$PHONE_PARENTHESES'           then 'Phone Parens'
        when '$PHONE_WORK'                  then 'Work Phone'
        when '$SUBJECT'                     then 'Subject'
        when '$TOPIC'                       then 'Topic'
        else
          raise "#{name} not recognized"
        end
      else
        name
      end
    end

    def options_str
      Options.new(options[:options]).to_s + options_required_str
    end

    def options_required_str
      required.nil? ? "" : ", required: #{required}"
    end

    # Note this is a class for options[:options] not the entire set of options
    # that include options[:required], etc.
    class Options
      def initialize(options)
        @options = options
        if @options
          to_array
          strip
          escape_quotes
          hashify
        end
      end

      def to_s
        @options || ''
      end
      
    private
      def to_array
        @options = case @options
          when Array then @options
          when Hash then @options.keys
          when String then [@options]
        end
      end

      def strip
        @options.map! { |option| option.strip }
      end

      def escape_quotes
        @options.map! { |option| option.gsub('"', '\"') }
      end

      def hashify
        @options = ", options: [\"" + @options.join("\", \"") + "\"]"
      end
    end
  end
end
